Attribute VB_Name = "Module1"
Option Explicit

Public Type RECT
    Left As Long
    Top As Long
    Right As Long
    Bottom As Long
End Type
    
Declare Function GetWindowRect Lib "user32" (ByVal HWnd As Long, lpRect As RECT) As Long

Declare Function BringWindowToTop Lib "user32" (ByVal HWnd As Long) As Long

Declare Function SetWindowPos Lib "user32" (ByVal HWnd As Long, _
                                            ByVal hWndInsertAfter As Long, _
                                            ByVal X As Long, _
                                            ByVal Y As Long, _
                                            ByVal cx As Long, _
                                            ByVal cy As Long, _
                                            ByVal wFlags As Long) As Long

Global Const HWND_TOP = 0
Global Const HWND_TOPMOST = -1
Global Const HWND_NOTOPMOST = -2

Global Const SWP_NOZORDER = &H10
Global Const SWP_NOACTIVATE = &H10
Global Const SWP_SHOWWINDOW = &H40

Public StopProcess As Boolean


Public Sub Status(Optional Message As Variant, Optional Filename As Variant)
' Displays the status form with the specified message and filename.
' All arguments are optional and if it's called without any arguments
' then it dismisses the form.

    ' Check to see if a message was passed in the argument.
    If IsMissing(Message) Then
        ' Turn off the display of the form.
        frmStatus.Visible = False
    Else
        ' Set up the text to display.
        frmStatus.lblStatus.Caption = Filename
        frmStatus.lblProcessing.Caption = Message
        
        ' Position the form if it's not visible. If it's already displayed
        ' leave it where it is because the user may have repositioned it.
        If frmStatus.Visible = False Then
            CenterForm 0, frmStatus
        End If
        
        ' Display the form.
        frmStatus.Visible = True
        
        ' Force the form to the top of all other windows.
        Call WindowToTop(frmStatus.HWnd)
        
        ' Force a refresh.
        frmStatus.Refresh
        DoEvents
    End If
End Sub

Public Sub WindowToTop(HWnd As Long)
' Forces a window to the top and keeps it on top.

    Dim Rec As RECT
    
    Call GetWindowRect(HWnd, Rec)
    Call SetWindowPos(HWnd, HWND_TOPMOST, Rec.Left, Rec.Top, Rec.Right - Rec.Left, Rec.Bottom - Rec.Top, SWP_NOACTIVATE Or SWP_SHOWWINDOW)
End Sub


Sub CenterForm(ByVal hWndParent As Long, ByVal frmForm As Form)
' This routine will center a form on the screen.

    Dim iLeft As Long
    Dim iTop As Long
    Dim iMidX As Long
    Dim iMidY As Long
    Dim rcParent As RECT

    'Find the ideal center point
    If hWndParent = 0 Then
        ' No parent, center over the screen using the screen object
        iMidX = Screen.Width / 2
        iMidY = Screen.Height / 4
    Else
        ' Center within the form's parent
        Call GetWindowRect(hWndParent, rcParent)

        ' in calculating mid x it seems to me that we should left*twipsX and right*twipsX
        ' rather than right*twipsY
        iMidX = ((rcParent.Left * Screen.TwipsPerPixelX) + _
                 (rcParent.Right * Screen.TwipsPerPixelY)) / 2
        iMidY = ((rcParent.Top * Screen.TwipsPerPixelY) + _
                 (rcParent.Bottom * Screen.TwipsPerPixelY)) / 2

        ' If the application is maximized or the app for some reason returns all 0 in the
        ' rectangle type, then center on the screen
        If (rcParent.Left = 0 And rcParent.Right = 0 And _
            rcParent.Top = 0 And rcParent.Bottom = 0) Then
            iMidX = Screen.Width / 2
            iMidY = Screen.Height / 2
        End If
    End If


    ' Find the form's upper left based on that
    iLeft = iMidX - (frmForm.Width / 2)
    iTop = iMidY - (frmForm.Height / 2)

    ' If the form is outside the screen, move it inside
    If iLeft < 0 Then
        iLeft = 0
    ElseIf (iLeft + frmForm.Width) > Screen.Width Then
        iLeft = Screen.Width - frmForm.Width
    End If

    If iTop < 0 Then
        iTop = 0
    ElseIf (iTop + frmForm.Height) > Screen.Height Then
        iTop = Screen.Height - frmForm.Height
    End If

    ' Move the form to its new position
    frmForm.Move iLeft, iTop
End Sub


